@@ -4,15 +4,18 @@
 
 """OSX platform implementation."""
 
+import contextlib
 import errno
 import functools
 import os
+from socket import AF_INET
 from collections import namedtuple
 
 from . import _common
 from . import _psposix
 from . import _psutil_osx as cext
 from . import _psutil_posix as cext_posix
+from ._common import AF_INET6
 from ._common import conn_tmap
 from ._common import isfile_strict
 from ._common import memoize_when_activated
@@ -26,14 +29,13 @@ __extra__all__ = []
 
 
 # =====================================================================
-# --- constants
+# --- globals
 # =====================================================================
 
 
 PAGESIZE = os.sysconf("SC_PAGE_SIZE")
 AF_LINK = cext_posix.AF_LINK
 
-# http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
 TCP_STATUSES = {
     cext.TCPS_ESTABLISHED: _common.CONN_ESTABLISHED,
     cext.TCPS_SYN_SENT: _common.CONN_SYN_SENT,
@@ -82,22 +84,6 @@ pidtaskinfo_map = dict(
     volctxsw=7,
 )
 
-scputimes = namedtuple('scputimes', ['user', 'nice', 'system', 'idle'])
-
-svmem = namedtuple(
-    'svmem', ['total', 'available', 'percent', 'used', 'free',
-              'active', 'inactive', 'wired'])
-
-pmem = namedtuple('pmem', ['rss', 'vms', 'pfaults', 'pageins'])
-pfullmem = namedtuple('pfullmem', pmem._fields + ('uss', ))
-
-pmmap_grouped = namedtuple(
-    'pmmap_grouped',
-    'path rss private swapped dirtied ref_count shadow_depth')
-
-pmmap_ext = namedtuple(
-    'pmmap_ext', 'addr perms ' + ' '.join(pmmap_grouped._fields))
-
 # these get overwritten on "import psutil" from the __init__.py file
 NoSuchProcess = None
 ZombieProcess = None
@@ -105,6 +91,30 @@ AccessDenied = None
 TimeoutExpired = None
 
 
+# =====================================================================
+# --- named tuples
+# =====================================================================
+
+
+# psutil.cpu_times()
+scputimes = namedtuple('scputimes', ['user', 'nice', 'system', 'idle'])
+# psutil.virtual_memory()
+svmem = namedtuple(
+    'svmem', ['total', 'available', 'percent', 'used', 'free',
+              'active', 'inactive', 'wired'])
+# psutil.Process.memory_info()
+pmem = namedtuple('pmem', ['rss', 'vms', 'pfaults', 'pageins'])
+# psutil.Process.memory_full_info()
+pfullmem = namedtuple('pfullmem', pmem._fields + ('uss', ))
+# psutil.Process.memory_maps(grouped=True)
+pmmap_grouped = namedtuple(
+    'pmmap_grouped',
+    'path rss private swapped dirtied ref_count shadow_depth')
+# psutil.Process.memory_maps(grouped=False)
+pmmap_ext = namedtuple(
+    'pmmap_ext', 'addr perms ' + ' '.join(pmmap_grouped._fields))
+
+
 # =====================================================================
 # --- memory
 # =====================================================================
@@ -185,6 +195,7 @@ disk_io_counters = cext.disk_io_counters
 
 
 def disk_partitions(all=False):
+    """Return mounted disk partitions as a list of namedtuples."""
     retlist = []
     partitions = cext.disk_partitions()
     for partition in partitions:
@@ -209,6 +220,7 @@ net_if_addrs = cext_posix.net_if_addrs
 
 
 def net_connections(kind='inet'):
+    """System-wide network connections."""
     # Note: on OSX this will fail with AccessDenied unless
     # the process is owned by root.
     ret = []
@@ -250,15 +262,16 @@ def boot_time():
 
 
 def users():
+    """Return currently connected users as a list of namedtuples."""
     retlist = []
     rawlist = cext.users()
     for item in rawlist:
-        user, tty, hostname, tstamp = item
+        user, tty, hostname, tstamp, pid = item
         if tty == '~':
             continue  # reboot or shutdown
         if not tstamp:
             continue
-        nt = _common.suser(user, tty or None, hostname or None, tstamp)
+        nt = _common.suser(user, tty or None, hostname or None, tstamp, pid)
         retlist.append(nt)
     return retlist
 
@@ -281,22 +294,40 @@ def wrap_exceptions(fun):
         try:
             return fun(self, *args, **kwargs)
         except OSError as err:
-            if self.pid == 0:
-                if 0 in pids():
-                    raise AccessDenied(self.pid, self._name)
-                else:
-                    raise
             if err.errno == errno.ESRCH:
-                if not pid_exists(self.pid):
-                    raise NoSuchProcess(self.pid, self._name)
-                else:
-                    raise ZombieProcess(self.pid, self._name, self._ppid)
+                raise NoSuchProcess(self.pid, self._name)
             if err.errno in (errno.EPERM, errno.EACCES):
                 raise AccessDenied(self.pid, self._name)
             raise
     return wrapper
 
 
+@contextlib.contextmanager
+def catch_zombie(proc):
+    """There are some poor C APIs which incorrectly raise ESRCH when
+    the process is still alive or it's a zombie, or even RuntimeError
+    (those who don't set errno). This is here in order to solve:
+    https://github.com/giampaolo/psutil/issues/1044
+    """
+    try:
+        yield
+    except (OSError, RuntimeError) as err:
+        if isinstance(err, RuntimeError) or err.errno == errno.ESRCH:
+            try:
+                # status() is not supposed to lie and correctly detect
+                # zombies so if it raises ESRCH it's true.
+                status = proc.status()
+            except NoSuchProcess:
+                raise err
+            else:
+                if status == _common.STATUS_ZOMBIE:
+                    raise ZombieProcess(proc.pid, proc._name, proc._ppid)
+                else:
+                    raise AccessDenied(proc.pid, proc._name)
+        else:
+            raise
+
+
 class Process(object):
     """Wrapper class around underlying C implementation."""
 
@@ -317,7 +348,8 @@ class Process(object):
     @memoize_when_activated
     def _get_pidtaskinfo(self):
         # Note: should work for PIDs owned by user only.
-        ret = cext.proc_pidtaskinfo_oneshot(self.pid)
+        with catch_zombie(self):
+            ret = cext.proc_pidtaskinfo_oneshot(self.pid)
         assert len(ret) == len(pidtaskinfo_map)
         return ret
 
@@ -336,19 +368,18 @@ class Process(object):
 
     @wrap_exceptions
     def exe(self):
-        return cext.proc_exe(self.pid)
+        with catch_zombie(self):
+            return cext.proc_exe(self.pid)
 
     @wrap_exceptions
     def cmdline(self):
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._name)
-        return cext.proc_cmdline(self.pid)
+        with catch_zombie(self):
+            return cext.proc_cmdline(self.pid)
 
     @wrap_exceptions
     def environ(self):
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._name)
-        return parse_environ_block(cext.proc_environ(self.pid))
+        with catch_zombie(self):
+            return parse_environ_block(cext.proc_environ(self.pid))
 
     @wrap_exceptions
     def ppid(self):
@@ -357,7 +388,8 @@ class Process(object):
 
     @wrap_exceptions
     def cwd(self):
-        return cext.proc_cwd(self.pid)
+        with catch_zombie(self):
+            return cext.proc_cwd(self.pid)
 
     @wrap_exceptions
     def uids(self):
@@ -407,7 +439,7 @@ class Process(object):
             rawtuple[pidtaskinfo_map['cpuutime']],
             rawtuple[pidtaskinfo_map['cpustime']],
             # children user / system times are not retrievable (set to 0)
-            0, 0)
+            0.0, 0.0)
 
     @wrap_exceptions
     def create_time(self):
@@ -430,7 +462,8 @@ class Process(object):
         if self.pid == 0:
             return []
         files = []
-        rawlist = cext.proc_open_files(self.pid)
+        with catch_zombie(self):
+            rawlist = cext.proc_open_files(self.pid)
         for path, fd in rawlist:
             if isfile_strict(path):
                 ntuple = _common.popenfile(path, fd)
@@ -443,13 +476,19 @@ class Process(object):
             raise ValueError("invalid %r kind argument; choose between %s"
                              % (kind, ', '.join([repr(x) for x in conn_tmap])))
         families, types = conn_tmap[kind]
-        rawlist = cext.proc_connections(self.pid, families, types)
+        with catch_zombie(self):
+            rawlist = cext.proc_connections(self.pid, families, types)
         ret = []
         for item in rawlist:
             fd, fam, type, laddr, raddr, status = item
             status = TCP_STATUSES[status]
             fam = sockfam_to_enum(fam)
             type = socktype_to_enum(type)
+            if fam in (AF_INET, AF_INET6):
+                if laddr:
+                    laddr = _common.addr(*laddr)
+                if raddr:
+                    raddr = _common.addr(*raddr)
             nt = _common.pconn(fd, fam, type, laddr, raddr, status)
             ret.append(nt)
         return ret
@@ -458,7 +497,8 @@ class Process(object):
     def num_fds(self):
         if self.pid == 0:
             return 0
-        return cext.proc_num_fds(self.pid)
+        with catch_zombie(self):
+            return cext.proc_num_fds(self.pid)
 
     @wrap_exceptions
     def wait(self, timeout=None):
@@ -469,11 +509,13 @@ class Process(object):
 
     @wrap_exceptions
     def nice_get(self):
-        return cext_posix.getpriority(self.pid)
+        with catch_zombie(self):
+            return cext_posix.getpriority(self.pid)
 
     @wrap_exceptions
     def nice_set(self, value):
-        return cext_posix.setpriority(self.pid, value)
+        with catch_zombie(self):
+            return cext_posix.setpriority(self.pid, value)
 
     @wrap_exceptions
     def status(self):
@@ -483,7 +525,8 @@ class Process(object):
 
     @wrap_exceptions
     def threads(self):
-        rawlist = cext.proc_threads(self.pid)
+        with catch_zombie(self):
+            rawlist = cext.proc_threads(self.pid)
         retlist = []
         for thread_id, utime, stime in rawlist:
             ntuple = _common.pthread(thread_id, utime, stime)
@@ -492,4 +535,5 @@ class Process(object):
 
     @wrap_exceptions
     def memory_maps(self):
-        return cext.proc_memory_maps(self.pid)
+        with catch_zombie(self):
+            return cext.proc_memory_maps(self.pid)
