@@ -16,6 +16,7 @@ import time
 import traceback
 from contextlib import closing
 
+from psutil import AIX
 from psutil import BSD
 from psutil import FREEBSD
 from psutil import LINUX
@@ -65,7 +66,8 @@ class TestAvailability(unittest.TestCase):
         self.assertEqual(hasattr(psutil, "win_service_get"), WINDOWS)
 
     def test_PROCFS_PATH(self):
-        self.assertEqual(hasattr(psutil, "PROCFS_PATH"), LINUX or SUNOS)
+        self.assertEqual(hasattr(psutil, "PROCFS_PATH"),
+                         LINUX or SUNOS or AIX)
 
     def test_win_priority(self):
         ae = self.assertEqual
@@ -159,7 +161,7 @@ class TestAvailability(unittest.TestCase):
 
     def test_proc_memory_maps(self):
         hasit = hasattr(psutil.Process, "memory_maps")
-        self.assertEqual(hasit, False if OPENBSD or NETBSD else True)
+        self.assertEqual(hasit, False if OPENBSD or NETBSD or AIX else True)
 
 
 # ===================================================================
@@ -372,12 +374,14 @@ class TestFetchAllProcesses(unittest.TestCase):
         self.assertGreaterEqual(ret, 0)
 
     def ppid(self, ret, proc):
-        self.assertIsInstance(ret, int)
+        self.assertIsInstance(ret, (int, long))
         self.assertGreaterEqual(ret, 0)
 
     def name(self, ret, proc):
         self.assertIsInstance(ret, str)
-        assert ret
+        # on AIX, "<exiting>" processes don't have names
+        if not AIX:
+            assert ret
 
     def create_time(self, ret, proc):
         self.assertIsInstance(ret, float)
@@ -482,7 +486,7 @@ class TestFetchAllProcesses(unittest.TestCase):
         for value in ret:
             self.assertIsInstance(value, (int, long))
             self.assertGreaterEqual(value, 0)
-        if POSIX and ret.vms != 0:
+        if POSIX and not AIX and ret.vms != 0:
             # VMS is always supposed to be the highest
             for name in ret._fields:
                 if name != 'vms':
@@ -536,8 +540,8 @@ class TestFetchAllProcesses(unittest.TestCase):
             check_connection_ntuple(conn)
 
     def cwd(self, ret, proc):
-        self.assertIsInstance(ret, str)
-        if ret is not None:  # BSD may return None
+        if ret:     # 'ret' can be None or empty
+            self.assertIsInstance(ret, str)
             assert os.path.isabs(ret), ret
             try:
                 st = os.stat(ret)
@@ -607,7 +611,7 @@ class TestFetchAllProcesses(unittest.TestCase):
     def num_ctx_switches(self, ret, proc):
         assert is_namedtuple(ret)
         for value in ret:
-            self.assertIsInstance(value, int)
+            self.assertIsInstance(value, (int, long))
             self.assertGreaterEqual(value, 0)
 
     def rlimit(self, ret, proc):
