@@ -762,21 +762,25 @@ class TestSystemNetwork(unittest.TestCase):
                 # Not always reliable.
                 # self.assertEqual(stats.isup, 'RUNNING' in out, msg=out)
                 self.assertEqual(stats.mtu,
-                                 int(re.findall(r'MTU:(\d+)', out)[0]))
+                                 int(re.findall(r'(?i)MTU[: ](\d+)', out)[0]))
 
     @retry_before_failing()
     def test_net_io_counters(self):
         def ifconfig(nic):
             ret = {}
             out = sh("ifconfig %s" % name)
-            ret['packets_recv'] = int(re.findall(r'RX packets:(\d+)', out)[0])
-            ret['packets_sent'] = int(re.findall(r'TX packets:(\d+)', out)[0])
-            ret['errin'] = int(re.findall(r'errors:(\d+)', out)[0])
-            ret['errout'] = int(re.findall(r'errors:(\d+)', out)[1])
-            ret['dropin'] = int(re.findall(r'dropped:(\d+)', out)[0])
-            ret['dropout'] = int(re.findall(r'dropped:(\d+)', out)[1])
-            ret['bytes_recv'] = int(re.findall(r'RX bytes:(\d+)', out)[0])
-            ret['bytes_sent'] = int(re.findall(r'TX bytes:(\d+)', out)[0])
+            ret['packets_recv'] = int(
+                re.findall(r'RX packets[: ](\d+)', out)[0])
+            ret['packets_sent'] = int(
+                re.findall(r'TX packets[: ](\d+)', out)[0])
+            ret['errin'] = int(re.findall(r'errors[: ](\d+)', out)[0])
+            ret['errout'] = int(re.findall(r'errors[: ](\d+)', out)[1])
+            ret['dropin'] = int(re.findall(r'dropped[: ](\d+)', out)[0])
+            ret['dropout'] = int(re.findall(r'dropped[: ](\d+)', out)[1])
+            ret['bytes_recv'] = int(
+                re.findall(r'RX (?:packets \d+ +)?bytes[: ](\d+)', out)[0])
+            ret['bytes_sent'] = int(
+                re.findall(r'TX (?:packets \d+ +)?bytes[: ](\d+)', out)[0])
             return ret
 
         nio = psutil.net_io_counters(pernic=True, nowrap=False)
@@ -1322,7 +1326,9 @@ class TestSensorsBattery(unittest.TestCase):
         # Emulate a case where energy_full file does not exist.
         # Expected fallback on /capacity.
         def open_mock(name, *args, **kwargs):
-            if name.startswith("/sys/class/power_supply/BAT0/energy_full"):
+            energy_full = "/sys/class/power_supply/BAT0/energy_full"
+            charge_full = "/sys/class/power_supply/BAT0/charge_full"
+            if name.startswith(energy_full) or name.startswith(charge_full):
                 raise IOError(errno.ENOENT, "")
             elif name.startswith("/sys/class/power_supply/BAT0/capacity"):
                 return io.BytesIO(b"88")
