@@ -21,6 +21,7 @@ import types
 
 import psutil
 
+from psutil import AIX
 from psutil import BSD
 from psutil import LINUX
 from psutil import NETBSD
@@ -47,9 +48,10 @@ from psutil.tests import HAS_MEMORY_MAPS
 from psutil.tests import HAS_PROC_CPU_NUM
 from psutil.tests import HAS_PROC_IO_COUNTERS
 from psutil.tests import HAS_RLIMIT
+from psutil.tests import HAS_THREADS
 from psutil.tests import mock
 from psutil.tests import PYPY
-from psutil.tests import PYTHON
+from psutil.tests import PYTHON_EXE
 from psutil.tests import reap_children
 from psutil.tests import retry_before_failing
 from psutil.tests import run_test_module_by_name
@@ -149,7 +151,7 @@ class TestProcess(unittest.TestCase):
         if POSIX:
             self.assertEqual(code, -signal.SIGKILL)
         else:
-            self.assertEqual(code, 0)
+            self.assertEqual(code, signal.SIGTERM)
         self.assertFalse(p.is_running())
 
         sproc = get_test_subprocess()
@@ -159,12 +161,12 @@ class TestProcess(unittest.TestCase):
         if POSIX:
             self.assertEqual(code, -signal.SIGTERM)
         else:
-            self.assertEqual(code, 0)
+            self.assertEqual(code, signal.SIGTERM)
         self.assertFalse(p.is_running())
 
         # check sys.exit() code
         code = "import time, sys; time.sleep(0.01); sys.exit(5);"
-        sproc = get_test_subprocess([PYTHON, "-c", code])
+        sproc = get_test_subprocess([PYTHON_EXE, "-c", code])
         p = psutil.Process(sproc.pid)
         self.assertEqual(p.wait(), 5)
         self.assertFalse(p.is_running())
@@ -173,7 +175,7 @@ class TestProcess(unittest.TestCase):
         # It is not supposed to raise NSP when the process is gone.
         # On UNIX this should return None, on Windows it should keep
         # returning the exit code.
-        sproc = get_test_subprocess([PYTHON, "-c", code])
+        sproc = get_test_subprocess([PYTHON_EXE, "-c", code])
         p = psutil.Process(sproc.pid)
         self.assertEqual(p.wait(), 5)
         self.assertIn(p.wait(), (5, None))
@@ -205,8 +207,8 @@ class TestProcess(unittest.TestCase):
             # to get None.
             self.assertEqual(ret2, None)
         else:
-            self.assertEqual(ret1, 0)
-            self.assertEqual(ret1, 0)
+            self.assertEqual(ret1, signal.SIGTERM)
+            self.assertEqual(ret1, signal.SIGTERM)
 
     def test_wait_timeout_0(self):
         sproc = get_test_subprocess()
@@ -225,7 +227,7 @@ class TestProcess(unittest.TestCase):
         if POSIX:
             self.assertEqual(code, -signal.SIGKILL)
         else:
-            self.assertEqual(code, 0)
+            self.assertEqual(code, signal.SIGTERM)
         self.assertFalse(p.is_running())
 
     def test_cpu_percent(self):
@@ -314,10 +316,10 @@ class TestProcess(unittest.TestCase):
 
         # test reads
         io1 = p.io_counters()
-        with open(PYTHON, 'rb') as f:
+        with open(PYTHON_EXE, 'rb') as f:
             f.read()
         io2 = p.io_counters()
-        if not BSD:
+        if not BSD and not AIX:
             self.assertGreater(io2.read_count, io1.read_count)
             self.assertEqual(io2.write_count, io1.write_count)
             if LINUX:
@@ -526,6 +528,7 @@ class TestProcess(unittest.TestCase):
         p = psutil.Process()
         self.assertGreater(p.num_handles(), 0)
 
+    @unittest.skipIf(not HAS_THREADS, 'not supported')
     def test_threads(self):
         p = psutil.Process()
         if OPENBSD:
@@ -550,6 +553,7 @@ class TestProcess(unittest.TestCase):
 
     @retry_before_failing()
     @skip_on_access_denied(only_if=OSX)
+    @unittest.skipIf(not HAS_THREADS, 'not supported')
     def test_threads_2(self):
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
@@ -687,12 +691,12 @@ class TestProcess(unittest.TestCase):
         sproc = get_test_subprocess()
         exe = psutil.Process(sproc.pid).exe()
         try:
-            self.assertEqual(exe, PYTHON)
+            self.assertEqual(exe, PYTHON_EXE)
         except AssertionError:
-            if WINDOWS and len(exe) == len(PYTHON):
+            if WINDOWS and len(exe) == len(PYTHON_EXE):
                 # on Windows we don't care about case sensitivity
                 normcase = os.path.normcase
-                self.assertEqual(normcase(exe), normcase(PYTHON))
+                self.assertEqual(normcase(exe), normcase(PYTHON_EXE))
             else:
                 # certain platforms such as BSD are more accurate returning:
                 # "/usr/local/bin/python2.7"
@@ -703,7 +707,7 @@ class TestProcess(unittest.TestCase):
                 ver = "%s.%s" % (sys.version_info[0], sys.version_info[1])
                 try:
                     self.assertEqual(exe.replace(ver, ''),
-                                     PYTHON.replace(ver, ''))
+                                     PYTHON_EXE.replace(ver, ''))
                 except AssertionError:
                     # Tipically OSX. Really not sure what to do here.
                     pass
@@ -712,7 +716,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(out, 'hey')
 
     def test_cmdline(self):
-        cmdline = [PYTHON, "-c", "import time; time.sleep(60)"]
+        cmdline = [PYTHON_EXE, "-c", "import time; time.sleep(60)"]
         sproc = get_test_subprocess(cmdline)
         try:
             self.assertEqual(' '.join(psutil.Process(sproc.pid).cmdline()),
@@ -722,27 +726,39 @@ class TestProcess(unittest.TestCase):
             # and Open BSD returns a truncated string.
             # Also /proc/pid/cmdline behaves the same so it looks
             # like this is a kernel bug.
-            if NETBSD or OPENBSD:
+            # XXX - AIX truncates long arguments in /proc/pid/cmdline
+            if NETBSD or OPENBSD or AIX:
                 self.assertEqual(
-                    psutil.Process(sproc.pid).cmdline()[0], PYTHON)
+                    psutil.Process(sproc.pid).cmdline()[0], PYTHON_EXE)
             else:
                 raise
 
     def test_name(self):
-        sproc = get_test_subprocess(PYTHON)
+        sproc = get_test_subprocess(PYTHON_EXE)
         name = psutil.Process(sproc.pid).name().lower()
         pyexe = os.path.basename(os.path.realpath(sys.executable)).lower()
         assert pyexe.startswith(name), (pyexe, name)
 
     # XXX
     @unittest.skipIf(SUNOS, "broken on SUNOS")
+    @unittest.skipIf(AIX, "broken on AIX")
     def test_prog_w_funky_name(self):
         # Test that name(), exe() and cmdline() correctly handle programs
         # with funky chars such as spaces and ")", see:
         # https://github.com/giampaolo/psutil/issues/628
+
+        def rm():
+            # Try to limit occasional failures on Appveyor:
+            # https://ci.appveyor.com/project/giampaolo/psutil/build/1350/
+            #     job/lbo3bkju55le850n
+            try:
+                safe_rmpath(funky_path)
+            except OSError:
+                pass
+
         funky_path = TESTFN + 'foo bar )'
         create_exe(funky_path)
-        self.addCleanup(safe_rmpath, funky_path)
+        self.addCleanup(rm)
         cmdline = [funky_path, "-c",
                    "import time; [time.sleep(0.01) for x in range(3000)];"
                    "arg1", "arg2", "", "arg3", ""]
@@ -847,7 +863,8 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(p.cwd(), os.getcwd())
 
     def test_cwd_2(self):
-        cmd = [PYTHON, "-c", "import os, time; os.chdir('..'); time.sleep(60)"]
+        cmd = [PYTHON_EXE, "-c",
+               "import os, time; os.chdir('..'); time.sleep(60)"]
         sproc = get_test_subprocess(cmd)
         p = psutil.Process(sproc.pid)
         call_until(p.cwd, "ret == os.path.dirname(os.getcwd())")
@@ -864,10 +881,9 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(len(initial), len(set(initial)))
 
         all_cpus = list(range(len(psutil.cpu_percent(percpu=True))))
-        # setting on travis doesn't seem to work (always return all
-        # CPUs on get):
-        # AssertionError: Lists differ: [0, 1, 2, 3, 4, 5, 6, ... != [0]
-        for n in all_cpus:
+        # Work around travis failure:
+        # https://travis-ci.org/giampaolo/psutil/builds/284173194
+        for n in all_cpus if not TRAVIS else initial:
             p.cpu_affinity([n])
             self.assertEqual(p.cpu_affinity(), [n])
             if hasattr(os, "sched_getaffinity"):
@@ -895,6 +911,8 @@ class TestProcess(unittest.TestCase):
         p.cpu_affinity(set(all_cpus))
         p.cpu_affinity(tuple(all_cpus))
 
+    # TODO: temporary, see: https://github.com/MacPython/psutil/issues/1
+    @unittest.skipIf(LINUX, "temporary")
     @unittest.skipIf(not HAS_CPU_AFFINITY, 'not supported')
     def test_cpu_affinity_errs(self):
         sproc = get_test_subprocess()
@@ -931,7 +949,7 @@ class TestProcess(unittest.TestCase):
 
         # another process
         cmdline = "import time; f = open(r'%s', 'r'); time.sleep(60);" % TESTFN
-        sproc = get_test_subprocess([PYTHON, "-c", cmdline])
+        sproc = get_test_subprocess([PYTHON_EXE, "-c", cmdline])
         p = psutil.Process(sproc.pid)
 
         for x in range(100):
@@ -1262,7 +1280,15 @@ class TestProcess(unittest.TestCase):
         # set methods
         succeed_or_zombie_p_exc(zproc.parent)
         if hasattr(zproc, 'cpu_affinity'):
-            succeed_or_zombie_p_exc(zproc.cpu_affinity, [0])
+            try:
+                succeed_or_zombie_p_exc(zproc.cpu_affinity, [0])
+            except ValueError as err:
+                if TRAVIS and LINUX and "not eligible" in str(err):
+                    # https://travis-ci.org/giampaolo/psutil/jobs/279890461
+                    pass
+                else:
+                    raise
+
         succeed_or_zombie_p_exc(zproc.nice, 0)
         if hasattr(zproc, 'ionice'):
             if LINUX:
@@ -1479,7 +1505,7 @@ class TestPopen(unittest.TestCase):
         # XXX this test causes a ResourceWarning on Python 3 because
         # psutil.__subproc instance doesn't get propertly freed.
         # Not sure what to do though.
-        cmd = [PYTHON, "-c", "import time; time.sleep(60);"]
+        cmd = [PYTHON_EXE, "-c", "import time; time.sleep(60);"]
         with psutil.Popen(cmd, stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE) as proc:
             proc.name()
@@ -1490,7 +1516,7 @@ class TestPopen(unittest.TestCase):
             proc.terminate()
 
     def test_ctx_manager(self):
-        with psutil.Popen([PYTHON, "-V"],
+        with psutil.Popen([PYTHON_EXE, "-V"],
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE,
                           stdin=subprocess.PIPE) as proc:
@@ -1504,7 +1530,7 @@ class TestPopen(unittest.TestCase):
         # subprocess.Popen()'s terminate(), kill() and send_signal() do
         # not raise exception after the process is gone. psutil.Popen
         # diverges from that.
-        cmd = [PYTHON, "-c", "import time; time.sleep(60);"]
+        cmd = [PYTHON_EXE, "-c", "import time; time.sleep(60);"]
         with psutil.Popen(cmd, stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE) as proc:
             proc.terminate()
